@@ -1,9 +1,9 @@
-/* $Id: x11-ui.c,v 1.5 2000/01/01 21:24:56 rsmith Exp $
+/* $Id: x11-ui.c,v 1.14 2001/06/02 10:51:32 rsmith Exp $
  * ------------------------------------------------------------------------
  * This file is part of xnetload, a program to monitor network traffic,
  * and display it in an X window.
  *
- * Copyright (C) 1997 - 1999  R.F. Smith <rsmith@xs4all.nl>
+ * Copyright (C) 1997 - 2001  R.F. Smith <rsmith@xs4all.nl>
  *
  * You can contact the author at the following address:
  *      email: rsmith@xs4all.nl
@@ -28,6 +28,37 @@
  * 
  * --------------------------------------------------------------------
  * $Log: x11-ui.c,v $
+ * Revision 1.14  2001/06/02 10:51:32  rsmith
+ * Updated copyright date.
+ *
+ * Revision 1.13  2001/04/18 17:44:36  rsmith
+ * Added the ZeroOnRequest flag.
+ *
+ * Revision 1.12  2000/09/27 18:59:29  rsmith
+ * Change kilo prefix from K to k.
+ *
+ * Revision 1.11  2000/09/26 19:50:31  rsmith
+ * Merged value-prefix patch from Jesper Dahlberg <jesper@swing.campus.luth.se>.
+ *
+ * Revision 1.10  2000/06/07 16:49:04  rsmith
+ * Fixed bug in get_out_value. Fix contributed by
+ * Jesper Dahlberg <jesper@swing.campus.luth.se>
+ *
+ * Revision 1.9  2000/04/14 20:22:12  rsmith
+ * Updated the copyright notice for 2000.
+ *
+ * Revision 1.8  2000/04/14 19:36:20  rsmith
+ * Added -h, -? and --help switches.
+ * Shortened the values strings.
+ * Shortened the interface string.
+ *
+ * Revision 1.7  2000/02/22 17:51:56  rsmith
+ * Updated fallback resources for greater width.
+ *
+ * Revision 1.6  2000/02/22 17:49:53  rsmith
+ * Implements scaling and totals feature. Initial patch by
+ * Paul Schilling <pfschill@bigfoot.com>, some changes by rsmith.
+ *
  * Revision 1.5  2000/01/01 21:24:56  rsmith
  * Release 1.7.1b3
  *
@@ -128,6 +159,12 @@
 #define XtCavg          "Avg"
 #define XtNkilobytes    "kilobytes"
 #define XtCkilobytes    "Kilobytes"
+#define XtNscale        "scale"
+#define XtCscale        "Scale"
+#define XtNhelp         "help"
+#define XtChelp         "Help"
+#define XtNzeroOnReset	"zeroOnReset"
+#define XtCzeroOnReset	"ZeroOnReset"
 
 /* application resource data type */
 typedef struct {
@@ -135,27 +172,50 @@ typedef struct {
   Boolean no_values;
   Boolean no_interface;
   Boolean kilobytes;
+  Boolean zeroonreset;
+  Boolean help;
   char *iface;
   int update;
   int avg;
+  int scale;
 } appdata_t;
 
 /* static variables */
 static appdata_t appdata;
-static char *in_alt[3] = {
-  "in:  %6.3f kB/s  max: %6.3f kB/s",
-  "in:  %5.1f p/s  max: %5.1f p/s"
+static char *in_alt[2] = {
+  "in:  %.1f %sB/s (%.1f %sB/s) [%.1f %sB]",
+  "in:  %.1f p/s  max: %.1f p/s"
 };
 static char *in_str = 0;
-static char *out_alt[3] = {
-  "out:  %6.3f kB/s  max: %6.3f kB/s",
-  "out:  %5.1f p/s  max: %5.1f p/s"
+static char *out_alt[2] = {
+  "out:  %.1f %sB/s (%.1f %sB/s) [%.1f %sB]",
+  "out:  %1.1f p/s  max: %.1f p/s"
 };
 static char *out_str = 0;
 
+/*
+ * Definition of the prefixes.  A thousand exa bytes will be
+ * a sufficient upper limit for quite a few years.  ;-)
+ */
+static char *prefix[] = {"", "k", "M", "G", "T", "P", "E"};
+
+struct prefixed_value {
+  float value;
+  char *prefix;
+};
+
 /* application resource list */
 static XtResource resources[] =
 {
+    {
+      XtNhelp,
+      XtChelp,
+      XtRBoolean,
+      sizeof(Boolean),
+      XtOffsetOf(appdata_t, help),
+      XtRBoolean,
+      (XtPointer) False
+    },
     {
       XtNnoValues,
       XtCnoValues,
@@ -201,6 +261,15 @@ static XtResource resources[] =
       XtRImmediate,
       (XtPointer) 1
     },      
+    {
+      XtNscale,
+      XtCscale,
+      XtRInt,
+      sizeof(int),
+      XtOffsetOf(appdata_t, scale),
+      XtRImmediate,
+      (XtPointer) 1
+    },      
     {
       XtNavg,
       XtCavg,
@@ -218,12 +287,24 @@ static XtResource resources[] =
       XtOffsetOf(appdata_t, kilobytes),
       XtRBoolean,
       (XtPointer) False
+    },
+    {
+      XtNzeroOnReset,
+      XtCzeroOnReset,
+      XtRBoolean,
+      sizeof(Boolean),
+      XtOffsetOf(appdata_t, zeroonreset),
+      XtRBoolean,
+      (XtPointer) False
     }
 };
 
 /* command line options */
 static XrmOptionDescRec options[] =
 {
+  {"-?", "*help", XrmoptionNoArg, "True"},
+  {"-h", "*help", XrmoptionNoArg, "True"},
+  {"--help", "*help", XrmoptionNoArg, "True"},
   {"-if", "*interface", XrmoptionSepArg, NULL},
   {"-interface", "*interface", XrmoptionSepArg, NULL},
   {"-ni", "*noInterface", XrmoptionNoArg, "True"},
@@ -237,7 +318,11 @@ static XrmOptionDescRec options[] =
   {"-a", "*avg", XrmoptionSepArg, NULL},
   {"-average", "*avg", XrmoptionSepArg, NULL},
   {"-kb", "*kilobytes", XrmoptionNoArg, "True"},
-  {"-kilobytes", "*kilobytes", XrmoptionNoArg, "True"}
+  {"-kilobytes", "*kilobytes", XrmoptionNoArg, "True"},
+  {"-s", "*scale", XrmoptionSepArg, NULL},
+  {"-scale", "*scale", XrmoptionSepArg, NULL},
+  {"-zr", "*zeroOnReset", XrmoptionNoArg, "True"},
+  {"-zeroonreset", "*zeroOnReset", XrmoptionNoArg, "True"}
 };
 
 /* time at which the program was started  */
@@ -251,8 +336,8 @@ static Widget toplevel, paned, interface, in, out, str_in, str_out;
 static String fallback_resources[] =
 {
   "*Label*background: LightGray",
-  "*Label*width: 260",
-  "*StripChart*width: 260",
+  "*Label*width: 280",
+  "*StripChart*width: 280",
   "*StripChart*height: 30",
   NULL
 };
@@ -289,6 +374,9 @@ get_in_value(Widget w, XtPointer client_data, XtPointer value);
 void
 get_out_value(Widget w, XtPointer client_data, XtPointer value);
 
+/* Determines appropriate prefix to use for 'value' */
+struct prefixed_value use_prefix(float value);
+
 /* Main procedure. Checks the arguments and aborts in case of error. 
  * Otherwise it creates and maps the widgets and goes into the message
  * processing loop.
@@ -311,10 +399,14 @@ int main(int argc, char *argv[])
                               resources,
                               XtNumber(resources),
                               NULL);
-#ifdef DEBUG
-  printf("appdata.update %d\n", appdata.update);
-  printf("appdata.avg %d\n", appdata.avg);
-#endif
+
+
+  /* check for help flags */
+  if (appdata.help == True) {
+      print_help();
+      return 1;
+  }
+
   /* check if interface was given */
   if (appdata.iface == 0) {
     if (argc > 1) {
@@ -326,6 +418,7 @@ int main(int argc, char *argv[])
       return 1;
     }
   }
+
   /* Check the command line arguments. */
   if (appdata.avg < 1) {
     report_error("Average count must be > 0");
@@ -333,6 +426,9 @@ int main(int argc, char *argv[])
   if (appdata.update < 1) {
     report_error("Update time must be > 0");
   }
+  if (appdata.scale < 1) {
+    report_error("Scale must be > 1");
+  }
 
   /* Initialize the data gathering process. */
   initialize(appdata.iface, appdata.avg);
@@ -416,14 +512,15 @@ int main(int argc, char *argv[])
 
 void print_help()
 {
-  fprintf(stderr, "xnetload "VERSION", Copyright (C) 1997-1999 R.F. Smith\n");
+  fprintf(stderr, "xnetload "VERSION", Copyright (C) 1997-2001 R.F. Smith\n");
   fprintf(stderr, "Usage: xnetload [Xt args] [options] [interface]\n");
   fprintf(stderr,
           "xnetload understands all Xt standard command-line options.\n");
   fprintf(stderr, "See X(1) for details. ");
-  fprintf(stderr, "Additional options are as follows:\n\n");
+  fprintf(stderr, "Additional options are as follows:\n");
   fprintf(stderr, "option name       option value\n");
   fprintf(stderr, "-----------       ------------\n");
+  fprintf(stderr, "-?. -h, --help    Display this help.\n");
   fprintf(stderr, "-if <name>,\n");
   fprintf(stderr, "-interface <name> Any device from /proc/net/dev.\n");
   fprintf(stderr, "-nc, -nocharts    Don't use charts.\n");
@@ -432,8 +529,12 @@ void print_help()
   fprintf(stderr, "-kb, -kilobytes   Display count in kilobytes.\n");
   fprintf(stderr, "-u,  -update      Number of seconds between screen.\n");
   fprintf(stderr, "                  updates. (default is 1).\n");
+  fprintf(stderr, "-s,  -scale       Number to scale kilobyte chart by.\n");
   fprintf(stderr, "-a,  -average     Number of samples to average.\n");
-  fprintf(stderr, "                  (default is 5).\n\n");
+  fprintf(stderr, "                  (default is 5).\n");
+  fprintf(stderr, "-zr, -zeroonreset Zero counters when resetting after\n");
+  fprintf(stderr, "                  the interface has gone down\n");
+  fprintf(stderr, "                  (e.g. ppp).\n");
   
   fprintf(stderr,
           "The network interface to monitor can also be named on the\n");
@@ -442,14 +543,13 @@ void print_help()
   fprintf(stderr, "with previous versions. ");
   fprintf(stderr,
           "However, the value of the `-if' option has precedence.\n");
-  fprintf(stderr, "\n");
 }
 
 /* Set the value that the StripChart widget str_in should display. */
 void get_in_value(Widget w, XtPointer client_data, XtPointer value)
 {
   if (appdata.kilobytes) {
-    *(double *)value = (double)average.in/1024;
+    *(double *)value = (double)average.in/(1024*appdata.scale);
   } else {
     if (average.in < 1.0) {
       *(double *)value = (double)0;
@@ -463,26 +563,25 @@ void get_in_value(Widget w, XtPointer client_data, XtPointer value)
 void get_out_value(Widget w, XtPointer client_data, XtPointer value)
 {
   if (appdata.kilobytes) {
-    *(double *)value = (double)average.out/1024;
+    *(double *)value = (double)average.out/(1024*appdata.scale);
   } else {
-    if (average.in < 1.0) {
+    if (average.out < 1.0) {
       *(double *)value = (double)0;
     } else {
       *(double *)value = log10(average.out);
     }
   }
-
-/*    *(double *) value = (double)avg_c.out; */
 }
 
 void refresh(XtPointer data, XtIntervalId * id)
 {
   long hr, min, sec;		/* time vars  */
+  struct prefixed_value pval_average, pval_max, pval_total;
   time_t newtime;
-  char *dev_str = "interface: %s  up: %2i:%02i:%02i";
+  char *dev_str = "%s up: %i:%02i:%02i";
   char buf[128];
   /* read data from /proc/net/dev */
-  update_avg(appdata.update);
+  update_avg(appdata.update, appdata.zeroonreset);
   /* get new time */
   time(&newtime);
   /* calculate the number of seconds passed since start */
@@ -502,14 +601,26 @@ void refresh(XtPointer data, XtIntervalId * id)
     XtVaSetValues(interface, XtNlabel, buf, NULL);
   }
   if (appdata.no_values == False) {
-    if (type == BYTES_TYPE)
-      sprintf(buf, in_str, average.in/1024, max.in/1024);
-    else
+    if (type == BYTES_TYPE) {
+      pval_average = use_prefix(average.in);
+      pval_max = use_prefix(max.in);
+      pval_total = use_prefix(total.in);
+      sprintf( buf, in_str
+             , pval_average.value, pval_average.prefix
+             , pval_max.value, pval_max.prefix
+             , pval_total.value, pval_total.prefix);
+    } else
       sprintf(buf, in_str, average.in, max.in);
     XtVaSetValues(in, XtNlabel, buf, NULL);
-    if (type == BYTES_TYPE)
-      sprintf(buf, out_str, average.out/1024, max.out/1024);
-    else
+    if (type == BYTES_TYPE) {
+      pval_average = use_prefix(average.out);
+      pval_max = use_prefix(max.out);
+      pval_total = use_prefix(total.out);
+      sprintf( buf, out_str
+             , pval_average.value, pval_average.prefix
+             , pval_max.value, pval_max.prefix
+             , pval_total.value, pval_total.prefix);
+    } else
       sprintf(buf, out_str, average.out, max.out);
     XtVaSetValues(out, XtNlabel, buf, NULL);
   }
@@ -524,3 +635,17 @@ void xclose(Widget w, XEvent * event, String * params, Cardinal * num)
   cleanup();
   exit(0);
 }
+
+struct prefixed_value use_prefix(float value)
+{
+  int i;
+  struct prefixed_value pval;
+
+  pval.value = value;
+  for (i = 0; pval.value > 1024.0; i++) {
+    pval.value /= 1024.0;
+  }
+  pval.prefix = prefix[i];
+
+  return pval;
+}
